/*
 * Haiku wrapper for ACE plugins.
 * Copyright (C) 2020 PulkoMandy <pulkomandy@pulkomandy.tk>
 *
 * Distributed under terms of the MIT license.
 */

#include <arpa/inet.h>
#include <stdio.h>

#include <Catalog.h>
#include <GridView.h>
#include <Joystick.h>
#include <LayoutBuilder.h>
#include <LocaleRoster.h>
#include <OptionPopUp.h>

#include <exec/types.h>

#include <acepansion/lib_header.h>

#include "../../Haiku/RichButton.h"

#define CATCOMP_NUMBERS
#include "generated/locale_strings.h" /* catalog string ids */

BJoystick* OpenJoystick(int index)
{
	char n[B_OS_NAME_LENGTH];
	BJoystick* j = new BJoystick();
	if (j->GetDeviceName(index, n) == B_OK)
		j->Open(n, true);
	return j;
}

void CloseJoystick(BJoystick* j)
{
	delete j;
}

void DeleteView(BView* view)
{
	BWindow* w = view->Window();
	if (w) {
		w->Lock();
		w->Quit();
	} else {
		delete view;
	}
}


void configureJoystick(struct PluginData*, ULONG, ULONG);

class JView: public BGridView
{
	public:
	JView(PluginData* data)
		: BGridView("plugin preferences")
		, myPlugin(data)
	{
	}

	void AttachedToWindow()
	{
		fMenu1->SetTarget(this);
		fMenu2->SetTarget(this);
	}

	void MessageReceived(BMessage* message)
	{
		switch (message->what)
		{
			case 'jys0':
			{
				int32 idx = message->FindInt32("be:value");
				configureJoystick(myPlugin, 0, idx);
				return;
			}
			case 'jys1':
			{
				int32 idx = message->FindInt32("be:value");
				configureJoystick(myPlugin, 1, idx);
				return;
			}
		}

		BGridView::MessageReceived(message);
	}

	BOptionPopUp* fMenu1;
	BOptionPopUp* fMenu2;
	PluginData* myPlugin;
};


BView* MakeView(STRPTR CYA_GroupJoystickPort[], PluginData* data, int j1, int j2)
{
	JView* v = new JView(data);
	v->SetToolTip(GetString(MSG_CYCLE_BUBBLE_HELP));

	BMessage* m = new BMessage('jys0');
	v->fMenu1 = new BOptionPopUp("Joystick selection", NULL, m);
	m = new BMessage('jys1');
	v->fMenu2 = new BOptionPopUp("Joystick selection", NULL, m);

	for (int i = 0; CYA_GroupJoystickPort[i] != NULL; i++) {
		v->fMenu1->AddOptionAt(CYA_GroupJoystickPort[i], i - 1, i);
		v->fMenu2->AddOptionAt(CYA_GroupJoystickPort[i], i - 1, i);
	}

	v->fMenu1->SetValue(j1);
	v->fMenu2->SetValue(j2);
	v->fMenu1->SetEnabled(false);
	v->fMenu2->SetEnabled(false);

	RichLabel* l1 = new RichLabel(NULL, GetString(MSG_LABEL_JOY0));
	RichLabel* l2 = new RichLabel(NULL, GetString(MSG_LABEL_JOY1));

	BLayoutBuilder::Grid<>(v)
		.Add(l1, 0, 0)
		.Add(v->fMenu1, 1, 0)
		.Add(l2, 0, 1)
		.Add(v->fMenu2, 1, 1);

	return v;
}

void EnableJoystickWindow(JView* view, bool activate)
{
	if (view->LockLooper() == true) {
		view->fMenu1->SetEnabled(activate);
		view->fMenu2->SetEnabled(activate);
		view->UnlockLooper();
	}
}
