/**************************************************************
 * Functions for easy and secure spawning/killing of subtasks *
 * (this code is based on MUI's SubTasks example)             *
 **************************************************************/

/* /// "Includes" */

#include <dos/dostags.h>
#include <dos/dosextens.h>

#include <proto/exec.h>
#include <proto/dos.h>

#include <acepansion/lib_header.h>

#include "subtasks.h"

/* /// */

/* /// "OS dependant tags definition" */

#ifndef __MORPHOS__
#define NP_CodeType TAG_IGNORE
#define CODETYPE_PPC TRUE
#endif

#ifndef __amigaos4__
#define NP_Child TAG_IGNORE
#endif

/* /// */

/* /// "Subtasks management functions" */

LONG SendSubTaskMsg(struct SubTask *st, WORD command, IPTR params)
{
	st->st_Message.stm_Message.mn_ReplyPort = st->st_Reply;
	st->st_Message.stm_Message.mn_Length    = sizeof(struct SubTaskMsg);
	st->st_Message.stm_Command              = command;
	st->st_Message.stm_Parameter            = params;
	st->st_Message.stm_Result               = 0;

	PutMsg(command==STC_STARTUP ? &((struct Process *)st->st_Task)->pr_MsgPort : st->st_Port, (struct Message *)&st->st_Message);
    WaitPort(st->st_Reply);
    GetMsg(st->st_Reply);

	return(st->st_Message.stm_Result);
}

struct SubTask * SpawnSubTask(CONST_STRPTR name, VOID (*func)(VOID), APTR data)
{
    struct Library *DOSBase = OpenLibrary(DOSNAME, 0L);
	struct SubTask *st;

    if(DOSBase)
    {
    	if((st = AllocVec(sizeof(struct SubTask),MEMF_PUBLIC|MEMF_CLEAR)))
    	{
    		if((st->st_Reply = CreateMsgPort()))
    		{
    			st->st_Data = data;

    			if((st->st_Task = (struct Task *)CreateNewProcTags(NP_CodeType   , CODETYPE_PPC, // MorphOS only
                                                                   NP_Child      , TRUE,         // AmigaOS4 only
                                                                   NP_Entry      , (IPTR)func,
                                                                   NP_Name       , (IPTR)name,
                                                                   NP_Input      , Input(),
                                                                   NP_CloseInput , FALSE,
                                                                   NP_Output     , Output(),
                                                                   NP_CloseOutput, FALSE,
                                                                   TAG_DONE)))
    			{
    				if(SendSubTaskMsg(st, STC_STARTUP, (IPTR)st))
    				{
    					return(st);
    				}
    			}
    			DeleteMsgPort(st->st_Reply);
    		}
    		FreeVec(st);
        }
        CloseLibrary(DOSBase);
    }

	return NULL;
}

VOID KillSubTask(struct SubTask *st)
{
	SendSubTaskMsg(st, STC_SHUTDOWN, (IPTR)st);
	DeleteMsgPort(st->st_Reply);
	FreeVec(st);
}

/* the following functions are called from the sub task */

VOID ExitSubTask(struct SubTask *st, struct SubTaskMsg *stm)
{
	if(st->st_Port)
		DeleteMsgPort(st->st_Port);

	/*
	** We reply after a Forbid() to make sure we're really gone
	** when the main task continues.
	*/
#ifndef __amigaos4__
    /*
    ** Under Amiga OS 4 the Forbid() call is superfluous in case the NP_Child
    ** tag is set to TRUE: in this case there will be the system itself to take care
    ** not to close the main process before our created sub-process.
    */
	Forbid();
#endif
	stm->stm_Result = FALSE;
	ReplyMsg((struct Message *)stm);
}

struct SubTask * InitSubTask(VOID)
{
	struct Task *me = FindTask(NULL);
	struct SubTask *st;
	struct SubTaskMsg *stm;

	/*
	** Wait for our startup message from the SpawnSubTask() function.
	*/

	WaitPort(&((struct Process *)me)->pr_MsgPort);
	stm  = (struct SubTaskMsg *)GetMsg(&((struct Process *)me)->pr_MsgPort);
	st   = (struct SubTask *)stm->stm_Parameter;

	if((st->st_Port = CreateMsgPort()))
	{
		/*
		** Reply startup message, everything ok.
		** Note that if the initialization fails, the code falls
		** through and replies the startup message with a stm_Result
		** of 0 after a Forbid(). This tells SpawnSubTask() that the
		** sub task failed to run.
		*/

		stm->stm_Result = TRUE;
		ReplyMsg((struct Message *)stm);
		return st;
	}
	else
	{
		ExitSubTask(st, stm);
		return NULL;
	}
}

/* /// */


