/* ACEpansion generic library.
 *
 * This code is compiled in all ACEpansion add-ons. This is the version for Haiku.
*/

#include <exec/types.h>
#include <exec/rawfmt.h>
#include <proto/exec.h>

#include <arpa/inet.h>
#include <Catalog.h>

#define CATCOMP_BLOCK
#include "generated/locale_strings.h" /* catcomp block */

#include "../../private/acepansion_protos.h"
#include "acepansion.h"

#define UNUSED __attribute__((unused))
extern "C" {
/*
** Library API
*/
struct ACEpansionPlugin * LibCreatePlugin(UNUSED struct LibBase *base, CONST_STRPTR *toolTypes, struct SignalSemaphore *sema)
{
    return CreatePlugin(toolTypes, sema);
}

VOID LibDeletePlugin(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    DeletePlugin(plugin);
}

BOOL LibActivatePlugin(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, BOOL activate)
{
    return ActivatePlugin(plugin, activate);
}

VOID LibReset(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    Reset(plugin);
}

VOID LibEmulate(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, ULONG *signals)
{
    Emulate(plugin, signals);
}

VOID LibWriteIO(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, USHORT port, UBYTE value)
{
    WriteIO(plugin, port, value);
}

VOID LibReadIO(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, USHORT port, UBYTE *value)
{
    ReadIO(plugin, port, value);
}

VOID LibGetAudio(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, SHORT *leftSample, SHORT *rightSample)
{
    GetAudio(plugin, leftSample, rightSample);
}

VOID LibPrinter(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, UBYTE data, BOOL strobe, BOOL *busy)
{
    Printer(plugin, data, strobe, busy);
}

VOID LibCursor(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    Cursor(plugin);
}

VOID LibAcknowledgeInterrupt(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, UBYTE *ivr)
{
    AcknowledgeInterrupt(plugin, ivr);
}

#if API_VERSION >= 2

VOID LibJoystick(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, BOOL com1, BOOL com2, UBYTE *ioData)
{
    Joystick(plugin, com1, com2, ioData);
}

#if API_VERSION >= 3

VOID LibReturnInterrupt(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    ReturnInterrupt(plugin);
}

#if API_VERSION >= 4

VOID LibLightDeviceDiodePulse(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    LightDeviceDiodePulse(plugin);
}

VOID LibLightDeviceButton(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, BOOL pressed)
{
    LightDeviceButton(plugin, pressed);
}

#if API_VERSION >= 5

STRPTR * LibGetPrefsPlugin(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, APTR pool)
{
    return GetPrefsPlugin(plugin, pool);
}

#endif
#endif
#endif
#endif
}

STRPTR GetString(LONG id)
{
    LONG   *l;
    UWORD  *w;
    STRPTR  builtin;

    static BCatalog* catalog = BLocaleRoster::Default()->GetCatalog();
    if (catalog) {
        char* tmp = (STRPTR)catalog->GetString(id);
        if (tmp && tmp[0])
            return tmp;
    }

	l = (LONG *)CatCompBlock;

    while(*l != (LONG)htonl(id) )
    {
        w = (UWORD *)((ULONG)l + 4);
        l = (LONG *)((ULONG)l + (ULONG)ntohs(*w) + 6);
    }
    builtin = (STRPTR)((ULONG)l + 6);

    return builtin;
}

/*
** Allocate and format a tooltype using the provided pool
*/
STRPTR MakeToolType(APTR pool, CONST_STRPTR key, CONST_STRPTR val)
{
    STRPTR templateString;
    STRPTR tooltype;
    ULONG count = 0;

    if(val)
        templateString = "%s=%s";
    else
        templateString ="%s";

    NewRawDoFmt(templateString, RAWFMTFUNC_COUNT, (STRPTR)&count, key, val);

    if((tooltype = (STRPTR)AllocPooled(pool, count)))
    {
        NewRawDoFmt(templateString, RAWFMTFUNC_STRING, tooltype, key, val);
    }

    return tooltype;
}

