/*
** ACEpansion generic library header
*/

#include <proto/exec.h>
#include <proto/locale.h>

#include <exec/resident.h>
#include <exec/libraries.h>
#include <exec/system.h>
#include <exec/interrupts.h>
#include <exec/rawfmt.h>
#include <libraries/query.h>
#include <clib/alib_protos.h>

#include <acepansion/lib_header.h>

#include "acepansion.h"
#define CATCOMP_BLOCK
#include "generated/locale_strings.h" /* catcomp block */

struct LibBase
{
	struct Library          LibNode;
	APTR                    SegList;
	struct SignalSemaphore  BaseLock;
	BOOL                    InitFlag;
};

extern const char VersionTag[];

struct Library *SysBase;
struct Library *LocaleBase;
struct Locale  *Locale;
struct Catalog *Catalog;

struct Library *libOpen(void);
ULONG libClose(void);
APTR libFlush(void);
ULONG libReserved(void);

struct Library *createLib(struct Library *unused, APTR segList, struct Library *sysBase);
struct LibBase *initLib(struct LibBase *base, APTR segList, struct Library *SysBase);
APTR expungeLib(struct LibBase *base);
BOOL allocLibResources(struct LibBase *base);
VOID freeLibResources(struct LibBase *base);

struct ACEpansionPlugin * LibCreatePlugin(struct LibBase *base, CONST_STRPTR *toolTypes, struct SignalSemaphore *sema);
VOID LibDeletePlugin(struct LibBase *base, struct ACEpansionPlugin *plugin);
BOOL LibActivatePlugin(struct LibBase *base, struct ACEpansionPlugin *plugin, BOOL status);
VOID LibReset(struct LibBase *base, struct ACEpansionPlugin *plugin);
VOID LibEmulate(struct LibBase *base, struct ACEpansionPlugin *plugin, ULONG *signals);
VOID LibWriteIO(struct LibBase *base, struct ACEpansionPlugin *plugin, USHORT port, UBYTE value);
VOID LibReadIO(struct LibBase *base, struct ACEpansionPlugin *plugin, USHORT port, UBYTE * value);
VOID LibGetAudio(struct LibBase *base, struct ACEpansionPlugin *plugin, SHORT * leftSample, SHORT *rightSample);
VOID LibPrinter(struct LibBase *base, struct ACEpansionPlugin *plugin, UBYTE data, BOOL strobe, BOOL *busy);
VOID LibCursor(struct LibBase *base, struct ACEpansionPlugin *plugin);
VOID LibAcknowledgeInterrupt(struct LibBase *base, struct ACEpansionPlugin *plugin, UBYTE *ivr);
#if API_VERSION >= 2
VOID LibJoystick(struct LibBase *base, struct ACEpansionPlugin *plugin, BOOL com1, BOOL com2, UBYTE *ioData);
#if API_VERSION >= 3
VOID LibReturnInterrupt(struct LibBase *base, struct ACEpansionPlugin *plugin);
#if API_VERSION >= 4
VOID LibLightDeviceDiodePulse(struct LibBase *base, struct ACEpansionPlugin *plugin);
VOID LibLightDeviceButton(struct LibBase *base, struct ACEpansionPlugin *plugin, BOOL pressed);
#if API_VERSION >= 5
STRPTR * LibGetPrefsPlugin(struct LibBase *base, struct ACEpansionPlugin *plugin, APTR pool);
#endif
#endif
#endif
#endif

const char libName[] = LIBNAME;

const struct TagItem rTags[] =
{
	{ QUERYINFOATTR_NAME     , (IPTR)libName        },
	{ QUERYINFOATTR_IDSTRING , (IPTR)&VersionTag[1] },
	{ QUERYINFOATTR_COPYRIGHT, (IPTR)COPYRIGHT      },
	{ QUERYINFOATTR_DATE     , (IPTR)DATE           },
	{ QUERYINFOATTR_VERSION  , VERSION              },
	{ QUERYINFOATTR_REVISION , REVISION             },
	{ QUERYINFOATTR_SUBTYPE  , QUERYSUBTYPE_LIBRARY },
	{ TAG_END,  0 }
};

struct Resident romTag =
{
	RTC_MATCHWORD,
	&romTag,
	&romTag + 1,
	RTF_EXTENDED | RTF_PPC,
	VERSION,
	NT_LIBRARY,
	0,
	(char *)libName,
	(char *)&VersionTag[1],
	(APTR)createLib,
	REVISION,
	(struct TagItem *)rTags
};

APTR jumpTable[] =
{
	(APTR)FUNCARRAY_BEGIN,
	(APTR)FUNCARRAY_32BIT_NATIVE,
	(APTR)libOpen,
	(APTR)libClose,
	(APTR)libFlush,
	(APTR)libReserved,
	(APTR)0xFFFFFFFF,
	(APTR)FUNCARRAY_32BIT_SYSTEMV,
	(APTR)LibCreatePlugin,
	(APTR)LibDeletePlugin,
	(APTR)LibActivatePlugin,
	(APTR)LibReset,
	(APTR)LibEmulate,
	(APTR)LibWriteIO,
	(APTR)LibReadIO,
	(APTR)LibGetAudio,
	(APTR)LibPrinter,
	(APTR)LibCursor,
	(APTR)LibAcknowledgeInterrupt,
#if API_VERSION >= 2
	(APTR)LibJoystick,
#if API_VERSION >= 3
	(APTR)LibReturnInterrupt,
#if API_VERSION >= 4
	(APTR)LibLightDeviceDiodePulse,
	(APTR)LibLightDeviceButton,
#if API_VERSION >= 5
	(APTR)LibGetPrefsPlugin,
#endif
#endif
#endif
#endif
	(APTR)0xFFFFFFFF,
	(APTR)FUNCARRAY_END
};

struct TagItem libTags[] =
{
	{ LIBTAG_FUNCTIONINIT, (IPTR)jumpTable             },
	{ LIBTAG_LIBRARYINIT , (IPTR)initLib               },
	{ LIBTAG_MACHINE     , MACHINE_PPC                 },
	{ LIBTAG_BASESIZE    , sizeof(struct LibBase)      },
	{ LIBTAG_SEGLIST     , 0                           },
	{ LIBTAG_TYPE        , NT_LIBRARY                  },
	{ LIBTAG_NAME        , 0                           },
	{ LIBTAG_IDSTRING    , 0                           },
	{ LIBTAG_FLAGS       , LIBF_CHANGED | LIBF_SUMUSED },
	{ LIBTAG_VERSION     , VERSION                     },
	{ LIBTAG_REVISION    , REVISION                    },
	{ LIBTAG_PUBLIC      , TRUE                        },
	{ TAG_END            , 0                           },
};

struct LibBase * initLib(struct LibBase *base, APTR segList, UNUSED struct Library *sysbase)
{
	InitSemaphore(&base->BaseLock);
	base->SegList = segList;
	return base;
}

struct Library * createLib(UNUSED struct Library *unused, APTR segList, struct Library *sysBase)
{
	SysBase = sysBase;

	libTags[4].ti_Data = (IPTR)segList;
	libTags[6].ti_Data = (IPTR)romTag.rt_Name;
	libTags[7].ti_Data = (IPTR)romTag.rt_IdString;

	return (NewCreateLibrary(libTags));
}

APTR expungeLib(struct LibBase *base)
{
	APTR segList = NULL;

	ObtainSemaphore(&base->BaseLock);

	if(base->LibNode.lib_OpenCnt == 0)
	{
		freeLibResources(base);
		Forbid();
		Remove((struct Node *)base);
		Permit();
		segList = base->SegList;
		FreeMem((UBYTE*)base - base->LibNode.lib_NegSize, base->LibNode.lib_NegSize + base->LibNode.lib_PosSize);
		base = NULL;
	}
	else
    {
        base->LibNode.lib_Flags |= LIBF_DELEXP;
    }

	if(base)
    {
        ReleaseSemaphore(&base->BaseLock);
    }

	return segList;
}


BOOL allocLibResources(UNUSED struct LibBase *base)
{
    BOOL success = FALSE;

    if((LocaleBase = OpenLibrary("locale.library", 0L)))
    {
    	if((Locale = OpenLocale(NULL)))
        {
    		Catalog = OpenCatalog(Locale, LIBNAME".catalog",
                OC_Version, VERSION,
                TAG_DONE);

            success = TRUE;
    	}
    }

    if(success)
    {
        success = InitResources();
    }

    return success;
}


VOID freeLibResources(UNUSED struct LibBase *base)
{
    if(LocaleBase)
    {
        if(Locale)
        {
            if(Catalog)
            {
                CloseCatalog(Catalog);
                Catalog = NULL;
            }

            CloseLocale(Locale);
            Locale = NULL;
        }
    }

	FreeResources();
}


struct Library * libOpen(void)
{
	struct LibBase *base = (struct LibBase *)REG_A6;
	struct Library *lib = (struct Library*)base;

	ObtainSemaphore(&base->BaseLock);

	if(!base->InitFlag)
	{
		if(allocLibResources(base))
        {
            base->InitFlag = TRUE;
        }
		else
		{
			freeLibResources(base);
			lib = NULL;
		}
	}

	if(lib)
	{
		base->LibNode.lib_Flags &= ~LIBF_DELEXP;
		base->LibNode.lib_OpenCnt++;
	}

	ReleaseSemaphore(&base->BaseLock);

	if(!lib)
    {
        expungeLib(base);
    }
	return lib;
}


ULONG libClose(void)
{
	struct LibBase *base = (struct LibBase *)REG_A6;
	ULONG ret = 0;

	ObtainSemaphore(&base->BaseLock);

	if(--base->LibNode.lib_OpenCnt == 0)
	{
		if(base->LibNode.lib_Flags & LIBF_DELEXP)
        {
            ret = (ULONG)expungeLib(base);
        }
	}

	if(ret == 0) ReleaseSemaphore(&base->BaseLock);
	return ret;
}


APTR libFlush(void)
{
	struct LibBase *base = (struct LibBase *)REG_A6;

	return(expungeLib(base));
}


ULONG libReserved(void)
{
	return 0;
}

/*
** Library API
*/
struct ACEpansionPlugin * LibCreatePlugin(UNUSED struct LibBase *base, CONST_STRPTR *toolTypes, struct SignalSemaphore *sema)
{
    return CreatePlugin(toolTypes, sema);
}

VOID LibDeletePlugin(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    DeletePlugin(plugin);
}

BOOL LibActivatePlugin(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, BOOL activate)
{
    return ActivatePlugin(plugin, activate);
}

VOID LibReset(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    Reset(plugin);
}

VOID LibEmulate(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, ULONG *signals)
{
    Emulate(plugin, signals);
}

VOID LibWriteIO(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, USHORT port, UBYTE value)
{
    WriteIO(plugin, port, value);
}

VOID LibReadIO(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, USHORT port, UBYTE *value)
{
    ReadIO(plugin, port, value);
}

VOID LibGetAudio(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, SHORT *leftSample, SHORT *rightSample)
{
    GetAudio(plugin, leftSample, rightSample);
}

VOID LibPrinter(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, UBYTE data, BOOL strobe, BOOL *busy)
{
    Printer(plugin, data, strobe, busy);
}

VOID LibCursor(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    Cursor(plugin);
}

VOID LibAcknowledgeInterrupt(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, UBYTE *ivr)
{
    AcknowledgeInterrupt(plugin, ivr);
}

#if API_VERSION >= 2

VOID LibJoystick(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, BOOL com1, BOOL com2, UBYTE *ioData)
{
    Joystick(plugin, com1, com2, ioData);
}

#if API_VERSION >= 3

VOID LibReturnInterrupt(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    ReturnInterrupt(plugin);
}

#if API_VERSION >= 4

VOID LibLightDeviceDiodePulse(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin)
{
    LightDeviceDiodePulse(plugin);
}

VOID LibLightDeviceButton(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, BOOL pressed)
{
    LightDeviceButton(plugin, pressed);
}

#if API_VERSION >= 5

STRPTR * LibGetPrefsPlugin(UNUSED struct LibBase *base, struct ACEpansionPlugin *plugin, APTR pool)
{
    return GetPrefsPlugin(plugin, pool);
}

#endif
#endif
#endif
#endif

/*
** Get a string out of the catalog from its id
*/
STRPTR GetString(LONG id)
{
    LONG   *l;
    UWORD  *w;
    STRPTR  builtin;
    STRPTR  fromCatalog;

	l = (LONG *)CatCompBlock;

    while(*l != id )
    {
        w = (UWORD *)((ULONG)l + 4);
        l = (LONG *)((ULONG)l + (ULONG)*w + 6);
    }
    builtin = (STRPTR)((ULONG)l + 6);

    fromCatalog = GetCatalogStr(Catalog, id, builtin);

    if(*fromCatalog == '\0')
        return builtin;
    else
        return fromCatalog;
}

/*
** Allocate and format a tooltype using the provided pool
*/
STRPTR MakeToolType(APTR pool, CONST_STRPTR key, CONST_STRPTR val)
{
    STRPTR template;
    STRPTR tooltype;
    ULONG count = 0;

    if(val)
        template = "%s=%s";
    else
        template ="%s";

    NewRawDoFmt(template, (APTR)RAWFMTFUNC_COUNT, (STRPTR)&count, key, val);

    if((tooltype = AllocPooled(pool, count)))
    {
        NewRawDoFmt(template, RAWFMTFUNC_STRING, tooltype, key, val);
    }

    return tooltype;
}

