/********************************************************************
 *                                                                  *
 * ACE Record (arec) to raw video command line                      *
 * Crappy version... Woops! ... I meant POSIX version :)            *
 * by OffseT of Futurs'                                             *
 *                                                                  *
 * GCC compilation:                                                 *
 * gcc -O3 ace-arec2raw.c -o ace-arec2raw                           *
 * strip ace-arec2raw                                               *
 *                                                                  *
 ** ace-arec2raw.c **************************************************/

/*
 TODO:
  -> Check that info chunk of all AREC parts are the same
*/

#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <arpa/inet.h>

#define MAKE_ID(a,b,c,d)  ((uint32_t)(a)<<24 | (uint32_t)(b)<<16 | (uint32_t)(c)<<8 | (uint32_t)(d))

#define RAW_AUDIO_TEMPLATE "%s.rawaudio"
#define RAW_VIDEO_TEMPLATE "%s.rawvideo"
#define FFMPEG_SCRIPT_TEMPLATE "%s.sh"

#define RC_SUCCESS  0
#define RC_FAIL     1

#define RLE_KEY 0x88888888

struct ProgArgs
{
    char *arecFileNameBase;
    char *rawFileNameBase;
};

struct VideoComputeData
{
    uint32_t frameCount;
    uint32_t currSync;
    uint64_t rate;
};

// INFO chunk content as stored in AREC files (big endian)
#pragma pack(1)
struct InfoChunk
{
    int32_t audioBufferSize;

    int32_t videoBufferSize;
    uint8_t videoPixelMode;
    uint8_t videoPixelSize;
    int32_t videoWidth;
    int32_t videoHeight;
    int32_t videoLineModulo;
};
#pragma pack()

// Pixel modes from ACE (videoPixelMode)
#define PIXEL_ARGB32  10 // 32 bit ARGB pixels
#define PIXEL_RGBA32  11 // 32 bit RGBA pixels
#define PIXEL_RGB16   20 // 16 bits RGB 5_6_5 pixels
#define PIXEL_RGB16PC 21 // 16 bits RGB 5_6_5 (PC) pixels

#define be2h_uint32_t(x) ntohl(x);
#define be2h_uint16_t(x) ntohs(x);
#define h2be_uint32_t(x) htonl(x);
#define h2be_uint16_t(x) htons(x);

// RLE routines template (warning: in/out sizes are in record, not in bytes)
#define UNCOMPRESS_RLE(size, type) \
static bool uncompressRLE##size(type inData[], uint32_t inSize, type outData[], uint32_t outSize, type key) \
{ \
    uint32_t outIdx = 0; \
    uint32_t inIdx = 0; \
 \
    while(outIdx < outSize && inIdx < inSize) \
    { \
        uint32_t i; \
        type value; \
        type count = 0; \
 \
        value = be2h_##type(inData[inIdx++]); \
 \
        if(value == key) \
        { \
            if(inIdx == inSize) break; \
            count = be2h_##type(inData[inIdx++]); \
 \
            if(count != 0) \
            { \
                if(inIdx == inSize) break; \
                value = be2h_##type(inData[inIdx++]); \
            } \
            else \
            { \
                count = 1; \
            } \
        } \
        else \
        { \
            count = 1; \
        } \
 \
        for(i=0; i<count && outIdx < outSize; i++) \
        { \
            outData[outIdx++] = h2be_##type(value); \
        } \
    } \
 \
    return outIdx == outSize && inIdx == inSize; \
}

// RLE uncompress implementations
UNCOMPRESS_RLE(16, uint16_t)
UNCOMPRESS_RLE(32, uint32_t)

// Decode ABUF chunk are store it in audio raw file
static int32_t exportRawAudio(struct InfoChunk *info, FILE *arecFile, int32_t chunkSize, FILE *rawAudioFile)
{
    int32_t rc = RC_SUCCESS;
    uint32_t *inAudioBuffer = malloc(chunkSize);
    uint32_t *outAudioBuffer = malloc(info->audioBufferSize);

    if(!inAudioBuffer || !outAudioBuffer)
    {
        rc = RC_FAIL;
    }
    else if(fread(inAudioBuffer, chunkSize, 1, arecFile) != 1)
    {
        rc = RC_FAIL;
    }
    else if(!uncompressRLE32(inAudioBuffer, chunkSize / sizeof(uint32_t),
                             outAudioBuffer, info->audioBufferSize / sizeof(uint32_t),
                             (uint32_t)RLE_KEY))
    {
        rc = RC_FAIL;
    }
    else if(fwrite(outAudioBuffer, info->audioBufferSize, 1, rawAudioFile) != 1)
    {
        rc = RC_FAIL;
    }

    free(inAudioBuffer);
    free(outAudioBuffer);

    return rc;
}

// Decode ABUF chunk are store it in audio raw file
static int32_t exportRawVideo(struct InfoChunk *info, FILE *arecFile, int32_t chunkSize, FILE *rawVideoFile, struct VideoComputeData *vdata)
{
    int32_t rc = RC_SUCCESS;
    void *inVideoBuffer = malloc(chunkSize);
    void *outVideoBuffer = malloc(info->videoBufferSize);

    if(!inVideoBuffer || !outVideoBuffer)
    {
        rc = RC_FAIL;
    }
    else if(fread(inVideoBuffer, chunkSize, 1, arecFile) != 1)
    {
        rc = RC_FAIL;
    }
    else if((info->videoPixelSize == sizeof(uint16_t) && !uncompressRLE16((uint16_t *)inVideoBuffer, chunkSize / info->videoPixelSize,
                                                                          (uint16_t *)outVideoBuffer, info->videoBufferSize / info->videoPixelSize,
                                                                          (uint16_t)RLE_KEY))
         || (info->videoPixelSize == sizeof(uint32_t) && !uncompressRLE32((uint32_t *)inVideoBuffer, chunkSize / info->videoPixelSize,
                                                                          (uint32_t *)outVideoBuffer, info->videoBufferSize / info->videoPixelSize,
                                                                          (uint32_t)RLE_KEY)))
    {
        rc = RC_FAIL;
    }
    else 
    {
        uint8_t *bufferCursor = outVideoBuffer;
        int32_t lineSize = info->videoWidth * info->videoPixelSize;
        int32_t line;

        for(line=0; line < info->videoHeight; line++)
        {
            if(fwrite(bufferCursor, lineSize, 1, rawVideoFile) != 1)
            {
                rc = RC_FAIL;
                break;
            }
            bufferCursor += info->videoLineModulo;
        }
        if(rc == RC_SUCCESS)
        {
            // Compute average frame rate
            vdata->rate = ((vdata->rate * (uint64_t)vdata->frameCount) + ((uint64_t)10000000000000000 / (uint64_t)vdata->currSync))
                        / ((uint64_t)vdata->frameCount + (uint64_t)1);
            vdata->frameCount++;
        }
    }

    free(inVideoBuffer);
    free(outVideoBuffer);

    return rc;
}

// Create ffmpeg encoder script
static int32_t createScript(struct InfoChunk *info, const char *outputFilesNameBase, const char *rawAudioFileName, const char *rawVideoFileName, struct VideoComputeData *vdata)
{
    int32_t rc = RC_SUCCESS;
    char *scriptFileName = malloc(strlen(outputFilesNameBase)+7/*.script*/+1/*0x00*/);

    if(!scriptFileName)
    {
        rc = RC_FAIL;
    }
    else
    {
        char *pixelFormatName = NULL;
        uint32_t inWidth, inHeight;
        uint32_t outWidth, outHeight;
        FILE *scriptFile;

        sprintf(scriptFileName, FFMPEG_SCRIPT_TEMPLATE, outputFilesNameBase);

        if(!(scriptFile = fopen(scriptFileName, "w+")))
        {
            rc = RC_FAIL;
        }
        else
        {
            inWidth = info->videoWidth;
            inHeight = info->videoHeight;
            outWidth = 768;
            outHeight = 576;

            switch(info->videoPixelMode)
            {
                case PIXEL_ARGB32:
                    pixelFormatName = "argb";
                    break;
                case PIXEL_RGBA32:
                    pixelFormatName = "rgba";
                    break;
                case PIXEL_RGB16PC:
                    pixelFormatName = "rgb565";
                    break;
                case PIXEL_RGB16:
                    pixelFormatName = "rgb565be";
                    break;
            }

            if(pixelFormatName)
            {
                fprintf(scriptFile, "#! /bin/sh\n"
                                    "ffmpeg -c:v rawvideo -f rawvideo -pix_fmt %s -s %dx%d -r %u.%010u -i %s"
                                          " -f s16be -ac 2 -ar 125000 -i %s"
                                          " -c:v mpeg4 -r 50.0801282051 -b:v 16M -s %dx%d -c:a mp2 -ac 2 -ar 48000 -b:a 320k -f avi %s.avi",
                                    pixelFormatName, inWidth, inHeight, (uint32_t)(vdata->rate / 10000000000), (uint32_t)(vdata->rate % 10000000000),
                                    rawVideoFileName,
                                    rawAudioFileName,
                                    outWidth, outHeight,
                                    outputFilesNameBase);
            }
            else
            {
                rc = RC_FAIL;
            }

            fclose(scriptFile);
        }
        free(scriptFileName);
    }

    return rc;
}

// Read chunks from AREC file(s) and generate audio and video raw file, as well as an encoding script for ffmpeg
static int32_t convertFile(const char *arecFileNameBase, const char *outputFilesNameBase)
{
    int32_t rc = RC_SUCCESS;
    bool processing = true;
    int32_t arecFilePartNbr = 0;
    FILE *arecFile;
    FILE *rawAudioFile = NULL;
    FILE *rawVideoFile = NULL;
    struct InfoChunk info;
    struct VideoComputeData vdata = { 0, 0, 0 };
    char *rawAudioFileName = malloc(strlen(outputFilesNameBase)+9/*.rawaudio*/+1/*0x00*/);
    char *rawVideoFileName = malloc(strlen(outputFilesNameBase)+9/*.rawvideo*/+1/*0x00*/);

    if(!rawAudioFileName || !rawVideoFileName)
    {
        rc = RC_FAIL;
    }
    else while(processing && rc == RC_SUCCESS)
    {
        int32_t dummy;
        bool readingInput = true;
        int32_t givenMainSize = 0;
        int32_t computedMainSize = 0;
        int32_t chunkCount = 0;

        // Check for AREC part top open (main or additional ones)
        if(arecFilePartNbr == 0)
        {
            // Open <file name> first part (no suffix)
            arecFile = fopen(arecFileNameBase, "r");
            if(arecFile)
            {
                // Open output files (only once, at first AREC file)
                sprintf(rawAudioFileName, RAW_AUDIO_TEMPLATE, outputFilesNameBase);
                sprintf(rawVideoFileName, RAW_VIDEO_TEMPLATE, outputFilesNameBase);

                if(!(rawAudioFile = fopen(rawAudioFileName, "w+")))
                {
                    rc = RC_FAIL;
                }
                if(!(rawVideoFile = fopen(rawVideoFileName, "w+")))
                {
                    rc = RC_FAIL;
                }
                if(rawAudioFile && rawVideoFile)
                {
                    printf("Processing file: %s...\n", arecFileNameBase);
                }
            }
        }
        else
        {
            // Open <file name>.xxxxx parts if any
            char *arecFileName = malloc(strlen(arecFileNameBase)+6/*.xxxxx*/+1/*0x00*/);

            sprintf(arecFileName, "%s.%05u", arecFileNameBase, arecFilePartNbr);
            arecFile = fopen(arecFileName, "r");
            if(arecFile)
            {
                printf("Processing file: %s...\n", arecFileName);
            }
            else
            {
                rc = RC_FAIL;
            }
            free(arecFileName);
        }

        // An AREC file should have been opened here (either the first one or additional parts)
        if(!arecFile)
        {
            if(rc == RC_FAIL
            && arecFilePartNbr != 0)
            {
                // All files processed, job done
                processing = false;
                rc = RC_SUCCESS;
            }
            else
            {
                rc = RC_FAIL;
            }
            continue;
        }

        // Go through the AREC file
        while(readingInput && rc == RC_SUCCESS)
        {
            int32_t chunkId;
            int32_t chunkSize;

            // Read chunk header
            if(fread(&chunkId, sizeof(chunkId), 1, arecFile) != 1
            || fread(&chunkSize, sizeof(chunkSize), 1, arecFile) != 1)
            {
                rc = RC_FAIL;
                break;
            }

            chunkId = ntohl(chunkId);
            chunkSize = ntohl(chunkSize);

            if(chunkCount == 0)
            {
                // First chunk shall be AREC (it contains all others)
                if(chunkId != MAKE_ID('A','R','E','C'))
                {
                    rc = RC_FAIL;
                    break;
                }
                givenMainSize = chunkSize;
                chunkCount++;
            }
            else
            {
                computedMainSize += sizeof(chunkId) + sizeof(chunkSize) + chunkSize;

                if(chunkCount == 1)
                {
                    // Second chunk shall be INFO
                    if(chunkId != MAKE_ID('I','N','F','O')
                    || sizeof(info) != chunkSize
                    || fread(&info, sizeof(info), 1, arecFile) != 1)
                    {
                        rc = RC_FAIL;
                        break;
                    }
                    info.audioBufferSize = ntohl(info.audioBufferSize);
                    info.videoBufferSize = ntohl(info.videoBufferSize);
                    info.videoWidth = ntohl(info.videoWidth);
                    info.videoHeight = ntohl(info.videoHeight);
                    info.videoLineModulo = ntohl(info.videoLineModulo);
                    chunkCount++;
                }
                else
                {
                    // Otherwise we have to check for other chunks
                    switch(chunkId)
                    {
                    case MAKE_ID('A','B','U','F'):
                        rc = exportRawAudio(&info, arecFile, chunkSize, rawAudioFile);
                        break;
                        //break;
                    case MAKE_ID('V','B','U','F'):
                        rc = exportRawVideo(&info, arecFile, chunkSize, rawVideoFile, &vdata);
                        break;
                    case MAKE_ID('S','Y','N','C'):
                        if(sizeof(vdata.currSync) != chunkSize
                        || fread(&vdata.currSync, sizeof(vdata.currSync), 1, arecFile) != 1)
                        {
                            rc = RC_FAIL;
                            break;
                        }
                        vdata.currSync = ntohl(vdata.currSync);
                        break;
                    default:
                        // Unknown chunk: skip it!
                        fseek(arecFile, chunkSize, SEEK_CUR);
                        break;
                    }
                }
            }

            // Check if the end on the main chunk (AREC) content in reached
            if(rc == RC_SUCCESS
            && givenMainSize == computedMainSize)
            {
                readingInput = false;
            }
        }

        // Check that we are actually at the end of the file
        if(rc == RC_SUCCESS
        && fread(&dummy, sizeof(dummy), 1, arecFile) == 1)
        {
            rc = RC_FAIL;
        }

        fclose(arecFile);
        arecFilePartNbr++;
    }

    // Now we can generate the encoding script
    if(rc == RC_SUCCESS)
    {
        rc = createScript(&info, outputFilesNameBase, rawAudioFileName, rawVideoFileName, &vdata);
    }

    if(rawAudioFile) fclose(rawAudioFile);
    if(rawVideoFile) fclose(rawVideoFile);

    free(rawAudioFileName);
    free(rawVideoFileName);

    return rc;
}

/*
 * Executable entry point
 *
 */
int main(int argc, char **argv)
{
    int32_t rc = RC_SUCCESS;
    int c;
    struct ProgArgs progArgs;

    // Default values
    progArgs.arecFileNameBase = NULL; // Mandatory
    progArgs.rawFileNameBase = NULL;

    while((c = getopt(argc, argv, "a:r:")) != -1) switch(c)
    {
    case 'a':
        progArgs.arecFileNameBase = optarg;
        break;
    case 'r':
        progArgs.rawFileNameBase = optarg;
        break;
    case '?':
        if(optopt == 'a' || optopt == 'r')
        {
            fprintf(stderr, "Option -%c requires an argument.\n", optopt);
        }
        else if(isprint(optopt))
        {
            fprintf(stderr, "Unknown option `-%c'.\n", optopt);
        }
        else
        {
            fprintf(stderr, "Unknown option character `\\x%x'.\n", optopt);
        }
        break;
    }

    if(progArgs.arecFileNameBase == NULL)
    {
        fprintf(stderr, "Syntax: %s -a <arec filename> [-r <raw filename base>]\n", argv[0]);
        rc = RC_FAIL;
    }
    else
    {
        // Use AREC file template if no raw template is provided
        if(!progArgs.rawFileNameBase)
        {
            progArgs.rawFileNameBase = progArgs.arecFileNameBase;
        }

        if((rc = convertFile(progArgs.arecFileNameBase, progArgs.rawFileNameBase)) == RC_SUCCESS)
        {
            printf("Generated files:\n"
                   ".Raw audio: "RAW_AUDIO_TEMPLATE"\n"
                   ".Raw video: "RAW_VIDEO_TEMPLATE"\n"
                   ".Encoder script: "FFMPEG_SCRIPT_TEMPLATE"\n",
                   progArgs.rawFileNameBase,
                   progArgs.rawFileNameBase,
                   progArgs.rawFileNameBase);
            printf("You can now use the generated script to encode a video from the raw files.\n");
        }
        else
        {
            fprintf(stderr, "%s: an error occured.\n", argv[0]);
        }
    }

    return rc;
}
