/* ImportBreakpoint.rexx - Script which is adding breakpoints into ACE from a breakpoint file generated by rasm
 *
 * Syntax: ImportBreakpoints.rexx <file name> rasm
 *
 * $VER: ImportBreakpoints.rexx 1.0 (10.07.20)  2020 by Philippe 'OffseT' Rimauro <offset@cpcscene.net>
 * Requires ACE 1.6 or later
 */

OPTIONS RESULTS
SIGNAL ON BREAK_C

ace_required_version="1.6"
ace_port="ACE.1"

appname="ACE Rexx"
LF="0A"x

PARSE ARG break_file file_type

/* Request arguments if not provided */
temp="T:symb.req"
pattern="#?"

IF file_type="" THEN DO
    ADDRESS 'COMMAND' 'RequestChoice >'temp' TITLE "'||appname||'" BODY "Please select which type of import should be done." GADGETS "rasm|Cancel"'
    IF OPEN(handle,temp,"R")~=0 THEN DO
        RESULT=READLN(handle)
        IF RESULT=0 THEN EXIT
        IF RESULT=1 THEN DO
            file_type="rasm"
            pattern="#?.brk"
        END
        CLOSE(handle)
    END
END

IF break_file="" THEN DO
    ADDRESS 'COMMAND' 'RequestFile >'temp' TITLE "Please choose the breakpoint file to import" PATTERN '||pattern
    IF RC~=0 THEN EXIT FatalError("No file selected!","Gasp!")
    IF OPEN(handle,temp,"R")~=0 THEN DO
        RESULT=READLN(handle)
        PARSE VAR RESULT '"'break_file'"'
        CLOSE(handle)
    END
END

/* Check arguments */
IF break_file="" | file_type="" THEN DO
    EXIT FatalError("Error in arguments!","Gasp!")
END
IF file_type~="rasm" THEN DO
    EXIT FatalError("Only file type rasm is supported!","Oops!")
END

/* Look for ACE running */
IF ~SHOW("P",ace_port) THEN DO
    EXIT FatalError("ACE CPC Emulator process not found!","Damned!")
END

/* ACE is not the default port */
ADDRESS(ace_port)

/* Check ACE version */
"VERSION"
IF CheckVersion(RESULT,ace_required_version)~=0 THEN DO
    EXIT FatalError("ACE CPC Emulator version" ace_required_version "or better is required!","I will update!")
END

/* Load required external libraries */
IF LoadLib("rexxsupport.library")~=0 THEN DO
    EXIT FatalError("rexxsupport.library not found!*NPlease install it.","Okay!")
END

/* Pause ACE while working */
"PAUSE"

/* Add breakpoints */
retval = AddBreakpoints_rasm(break_file)

IF retval~=0 THEN SAY "Break file '"break_file"' could not be added!"

/* Start ACE again */
"START"

EXIT


/*
 * Add breakpoints rasm (providing a breakpoint file generated by rasm)
 */
AddBreakpoints_rasm:
    PARSE ARG file_name

    IF OPEN(file_handle,file_name,"R")~=0 THEN DO
        DO WHILE ~EOF(file_handle)
            line=READLN(file_handle)
            PARSE VAR line "#"hex_addr
            IF hex_addr~="" THEN DO
                /*SAY "Found breakpoint a address &"hex_addr*/
                "ADDBREAKPOINT TYPE Execute ADDRESS 0x"hex_addr
                IF rc~=0 THEN DO
                    SAY "Line:" line
                    SAY "Hex_addr: " hex_addr
                END
            END
        END
        CLOSE(file_handle)
    END
    ELSE RETURN 20

    RETURN 0
 
/*
 * Version checker (ver.rev[beta])
 */
CheckVersion:
    PARSE ARG cv "." cr "beta" , rv "." rr "beta"

    IF cv<rv | (cv=rv & cr<rr) THEN DO
        RETURN 5
    END
    RETURN 0

/*
 * External lib loader
 */
LoadLib:
    PARSE ARG lib

    IF SHOW("L",lib)=0 THEN DO
        IF ADDLIB(lib,0,-30,0)=0 THEN DO
            RETURN 20
        END
    END
    RETURN 0

/*
 * Fatal error
 */
FatalError:
    PARSE ARG message,button

    ADDRESS "COMMAND" 'RequestChoice >NIL: TITLE "'||appname||'" BODY "'||message||'" GADGETS "'||button||'"'
    RETURN 20
